﻿using System;
using Ivi.DCPwr;

/////////////////////////////////////////////////////////////////////////////
// Name:        IviDcPwr_Simple_Example
//
// Purpose:     IviDcPwr sample program to demonstrate simple setting of Voltage, Current limit on two channels + measurement
//              using an R&S HMP4000 series power supplies in C#
//
//              The Example uses only the interchangeable Ivi.Driver and Ivi.DcPwr Interface.
//              Therefore, notice please, there is no reference to the Rohde & Scharz Hmp4000 instrument driver.
//              The link pointing to the Hmp4000 is the following:
//              - Logical Name (e.g. "myDCPwr"), here used in the method IviDCPwr.Create() which points to
//              - Driver Session (e.g. "myDCPwrSession") which has a property SoftwareModule "RohdeSchwarz.Hmp4000.Fx40"
//              (see the settings in Ivi Config Store configuration below in Prerequisities)
//              Therefore, when using Ivi-class drivers, you always need to use a configured logical name, not the direct resource name.
//              Otherwise, the link to the actual software module to use would be missing.
//
// Author:      Jiri Kominek
// Created:     07-November-2018
// Copyright:   (c) Rohde & Schwarz, Munich
/////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------
// Prerequisites
//
//   This sample program needs IVI Shared Components and IVI.NET Shared Components being installed.
//   If not yet done, please visit
//     http://www.ivifoundation.org/shared_components/Default.aspx
//
//   Download and install
//     - IVI Shared Components 2.6.0
//     - IVI.NET Shared Components 1.4.0
//     - Rohde & Schwarz Hmp4000 IVI.NET driver 2.6.2.0
//
//   32-bit: IviSharedComponents_2.6.0.exe
//           IviNetSharedComponents32_Fx20_1.4.0.exe
//           Hmp4000-ivi.net-x86-2_6_2_0.msi
//
//   64-bit: IviSharedComponents64_2.4.2.exe
//           IviNetSharedComponents64_Fx20_1.4.0.exe
//           Hmp4000-ivi.net-x64-2_6_2_0.msi
//
//   You always need to install the packages for your processor architecture

//   Configure your instrument in the IviConfigStore. The description below is for NI MAX:
//   - System -> IVI Drivers -> Driver Sessions -> CreateNew "myDCPwrSession"
//     - In the right tab "Software" (selectable at the bottom) choose SoftwareModule "RohdeSchwarz.Hmp4000.Fx40"
//     - In the right tab "Hardware" (selectable at the bottom) select/add one instrument in the HardwareAssets table e.g.:
//       Name = "HMP2020", ResourceDescriptor = "TCPIP::10.64.1.36::5025::SOCKET"
//   - System -> IVI Drivers -> Logical Names -> CreateNew "myDCPwr" - this is the name you refer to your instrument in the Create() method
//     - In right tab "General" choose Driver Session "myDCPwrSession"
//   - Save the IVI Configuration
//--------------------------------------------------------------------------

namespace IviDcPwr_Simple_Example
{
    class Program
    {
        static void Main(string[] args)
        {
            bool idQuery = true;
            bool resetDevice = true;
            IIviDCPwr driver = null;


            Console.Write("Initializing the instrument ... ");
            try
            {
                // Open the driver session, but only use the device-interchangeable IIviDcPwr interface
                driver = IviDCPwr.Create("myDCPwr", idQuery, resetDevice, "Simulate=False");
                Console.WriteLine("finished");
            }
            catch (Exception ex)
            {
                if (ex is Ivi.Driver.DriverClassCreationException
                    || ex is Ivi.Driver.SessionNotFoundException
                    || ex is Ivi.Driver.IOException)
                {
                    Console.WriteLine("\nException '{0}' occured.", ex.GetType());
                    Console.WriteLine("Check your instrument and your Ivi Config Store Configuration.\nDetails:");
                    Console.WriteLine(ex.Message);
                    Console.Write("\nPress any key to finish ... ");
                    Console.ReadKey();
                    return;
                }
                throw; // rethrow the exception if it is of other kind
            }

            try
            {
                // Selftest
                var selftestResult = driver.Utility.SelfTest();
                Console.WriteLine("Instrument Selftest: {0}", selftestResult.Message);

                // Identification
                var model = driver.Identity.InstrumentModel;
                Console.WriteLine("Instrument Model: {0}", model);
                var firmware = driver.Identity.InstrumentFirmwareRevision;
                Console.WriteLine("Instrument Firmware: {0}", firmware);
                var identifier = driver.Identity.Identifier;
                Console.WriteLine("Driver Identifier: {0}", identifier);

                // Settings for Channel 1
                Console.Write("Configuring Channel 1 ... ");
                driver.Outputs["1"].VoltageLevel = 1.12;
                driver.Outputs["1"].OvpLimit = 2.5;
                driver.Outputs["1"].CurrentLimit = 0.123;
                driver.Outputs["1"].Enabled = true;
                Console.WriteLine("finished");
                var measVoltageCh1 = driver.Outputs["1"].Measure(MeasurementType.Voltage);
                Console.WriteLine("Measured Voltage on Channel 1: {0:F3} V", measVoltageCh1);

                // Same settings for Channel 2
                Console.Write("Configuring Channel 2 ... ");
                driver.Outputs["2"].VoltageLevel = 2.34;
                driver.Outputs["2"].OvpLimit = 3.6;
                driver.Outputs["2"].Enabled = true;
                Console.WriteLine("finished");
                var measVoltageCh2 = driver.Outputs["2"].Measure(MeasurementType.Voltage);
                Console.WriteLine("Measured Voltage on Channel 2: {0:F3} V", measVoltageCh2);
            }
            //driver IO error
            catch (Ivi.Driver.IOException e)
            {
                Console.WriteLine("Driver error occured:");
                Console.WriteLine(e.Message);
            }

            //driver function is not supported
            catch (Ivi.Driver.OperationNotSupportedException e)
            {
                Console.WriteLine("Instrument doesn't support the function:");
                Console.WriteLine(e.Message);
            }

            //when a task takes longer than the defined timeout
            catch (Ivi.Driver.MaxTimeExceededException e)
            {
                Console.WriteLine("Operation took longer than the maximum defined time:");
                Console.WriteLine(e.Message);
            }

            //if the instrument returns an error in the error queue
            catch (Ivi.Driver.InstrumentStatusException e)
            {
                Console.WriteLine("Instrument system error occured:");
                Console.WriteLine(e.Message);
            }

            //if the instrument returns an error in the error queue
            catch (Ivi.Driver.SelectorNameException e)
            {
                Console.WriteLine("Invalid selector name used:");
                Console.WriteLine(e.Message);
            }

            finally
            {
                Console.Out.Write("\n\nPress any key to finish ... ");
                Console.ReadKey();
            }
        }
    }
}